#include <vector>
#include <map>
#include "stream.h"
using namespace std;

struct histogram {
    vector<long> leftpoints;
    vector<long> rightpoints;
    vector<double> heights;
    int k;
    double defaultVal;
};

struct error_breakdown {
    double errTotal;
    double errDefault;
    vector<double> errPieces;
    double heavySavings;
};

/*
Given a set of (x,y) pairs sorted by x values, find the k-piece histogram that best fits
the points. 
*/
struct histogram fitHistogramDP(const vector<long>& x, const vector<double>& y, int k, long minDomain, long maxDomain, bool useDefaultVal);

struct error_breakdown getApproxError(const struct histogram& hist, const vector<long>& heavyHitters, const vector<long>& support, const map<long, double>& density, const vector<double>& heavyHeights={});

/*
Runs the one pass algorithm, saves a plot of the result and returns the error.
*/
struct error_breakdown onePassAlgo(const Stream& stream, int space, int k, string plotName, bool useDefaultVal, bool verbose=true);

struct error_breakdown onePassAlgoSampling(const Stream& stream, int numSamples0, int numSamples1, int k, string plotName, bool useDefaultVal, bool verbose=true);

struct error_breakdown twoPassAlgo(const Stream& stream, int numSamples0, int k, string plotName, bool verbose);

double optHistogram(const Stream& stream, int k, bool supportAware, string plotName, bool verbose=false);

/*
Splits the domain into k fixed pieces and uses numSamples0/k L0 samples from each to approximate the median.
Calculates the error and saves a plot.
*/
double fixedIntervalAlgo(const Stream& stream, int numSamples0, int k, bool supportAware, string plotName);

/*
Calculate the error and save a plot of using the true median (1 piece histogram).
*/
double trueMedianAlgo(const Stream& stream, string plotName);

void plotApprox(const struct histogram& hist, const vector<long> heavyHitters, const vector<long> samplesL0, const Stream& stream, struct error_breakdown err, string outname, bool logscale=true);

void plotDensity(const Stream& stream, string outname, bool logscale=true);