#include "algorithms.h"
#include "stdio.h"
#include "stdlib.h"
#include <iostream>
#include <fstream>
#include <filesystem>
#include <ctime>

using namespace std;

void test(string dataname, int nrep) {
    vector<int> samples = {100, 200, 500, 1000, 2000, 5000};
    vector<int> ks = {5, 10};

    Stream stream = Stream(1);
    if (dataname == "caida-destip3") {
        stream = importCaida("destip", 3);
    } else if (dataname == "caida-destip2") {
        stream = importCaida("destip", 2);
        samples.pop_back();
    } else if (dataname == "taxi") {
        stream = importTaxi();
    } else if (dataname == "mcdonalds") {
        stream = importMcDonalds("lat");
        samples.pop_back();
    } else if (dataname == "warpeace") {
        stream = importWP();
        samples.pop_back();
    } else {
        cout << "UNKNOWN DATANAME" << endl;
        exit(0);
    }
    stream.printStats();

    int hash = (int) time(0);
    cout << hash << endl;
    string results_dir = "../results/" + dataname + "/"+to_string(hash);
    string plot_dir = "../plots/" + dataname + "/"+to_string(hash);
    if (!filesystem::create_directory(results_dir))
        exit(1);
    if (!filesystem::create_directory(plot_dir))
        exit(1);
    ofstream resultFile(results_dir+"/results.csv");
    resultFile << "s,k,t,opt_support,fixed_support,fixed_domain,1pass,2pass" << endl;

    plotDensity(stream, plot_dir + "/density.pdf");
    trueMedianAlgo(stream, plot_dir + "/median.pdf");

    // running main algo
    int i = 0;
    for (int sample: samples) {
        int j = 0;
        for (int k:  ks) {
            for (int t = 0; t < nrep; t++) {
                cout << "s" << sample << " k" << k << " t" << t << endl;
                resultFile << sample << "," <<  k << "," << t << ",";
                // baselines
                string plotName = "";
                double error;
                // opt_k support-aware
                if (stream.getSupportSize() < 3000 && i == 0 && t == 0) {
                    resultFile << ",";
                    // plotName = plot_dir + "/opt-support-k"+to_string(k)+"-s"+to_string(sample)+"-t"+to_string(t)+".pdf";
                    // error = optHistogram(stream, k, true, plotName);
                    // cout << "Optimal (support): " << error << endl;
                    // resultFile << error << ",";
                } else {
                    resultFile << ",";
                }
                // support aware
                if (t == 0)
                    plotName = plot_dir + "/fixed-support-k"+to_string(k)+"-s"+to_string(sample)+"-t"+to_string(t)+".pdf";
                error = fixedIntervalAlgo(stream, sample, k, true, plotName);
                cout << "Fixed (support): " << error << endl;
                resultFile << error << ",";
                // domain
                if (t == 0)
                    plotName = plot_dir + "/fixed-domain-k"+to_string(k)+"-s"+to_string(sample)+"-t"+to_string(t)+".pdf";
                error = fixedIntervalAlgo(stream, sample, k, false, plotName);
                cout << "Fixed (domain): " << error << endl;
                resultFile << error << ",";
                // one pass
                if (t == 0)
                    plotName = plot_dir+"/1pass-k"+to_string(k)+"-s"+to_string(sample)+"-t"+to_string(t)+".pdf";
                struct error_breakdown err = onePassAlgo(stream, sample, k, plotName, true, false);
                error = err.errTotal;
                cout << "One pass: " << error << endl;
                resultFile << error << ",";
                // two pass
                if (j == 0) {
                    if (t == 0)
                        plotName = plot_dir+"/2pass-k"+to_string(k)+"-s"+to_string(sample)+"-t"+to_string(t)+".pdf";
                    err = twoPassAlgo(stream, sample, k, plotName, false);
                    error = err.errTotal;
                    cout << "Two pass: " << error << endl;
                    resultFile << error << endl;
                } else {
                    resultFile << endl;
                }
                cout << endl << endl;
            }
            j++;
        }
        i++;
    }
}

int main(int argc, char* argv[]) {
    // taxi, mcdonalds, warpeace, caida-destip3
    assert(argc == 2);
    string dataname = argv[1];
    test(dataname, 10);
}