#include <vector>
#include <map>
#include <random>
using namespace std;

class Stream {
public:
    Stream(long domainSize);
    long getStreamLength() const;
    long getDomainSize() const;
    long getSupportSize() const;
    int setStream(const vector<long>& s);
    const vector<long>& getStream() const;
    const map<long, double>& getDensity() const;
    const vector<long>& getSupport() const;
    double getMinDensity() const;
    double getMaxDensity() const;
    void printStats() const;
    vector<long> sampleL0(long numSamples, const vector<long>& heavyHitters) const;
    vector<long> sampleL0FromRange(long numSamples, long left, long right, const vector<long>& heavyHitters) const;
    vector<long> sampleL1(long numSamples) const;
private:
    long n; // domain size
    long m; // stream length
    long supp; // support size
    vector<long> stream;
    map<long, double> density;
    vector<long> support;
    double minDensity;
    double maxDensity;
};

Stream uniformStream(long n);

Stream randomStream(long n, long m);

Stream lowerBoundStream(long n, long nElephant, long nMice, long sizeElephant);

Stream mixtureGaussianStream(long n, int k, int maxMean, double stdev, bool verbose=true); 

Stream importCaida(string type="destport", int nchunks=2, bool verbose=true);

Stream importMcDonalds(string type="lat", bool verbose=true);

Stream importStanford(bool verbose=true);

Stream importHamlet(bool verbose=true);

Stream importWP(bool verbose=true);

Stream importTaxi(bool verbose=true); 